#ifdef MODERRCORRB

double logDalecPost(Array1D<double> &chain, void *params) {

  //#define DEBUG
#ifdef DEBUG
  std::cout<<"In logDalecPost: "<<std::endl<<std::flush;
#endif
  llpars *pars   = (llpars *)params ;

  /* Extract parameters from params structure*/
  int     dayst  = pars -> dayst;
  int     dayen  = pars -> dayen;
  int     nday   = pars -> nday;
  int     ncycles= pars -> ncycles;

  double *drivers  = pars -> drivers  ;
  double *model    = pars -> model    ;
  double *obsdata  = pars -> obsdata  ;
  double *obssigma = pars -> obssigma ;
  
  int chainDim = chain.XSize() ;
  vector<double> parValues ;
  parValues.assign(chain.GetArrayPointer(),chain.GetArrayPointer()+chainDim-3);
  parValues.insert(parValues.begin()+8,25.0);
  dalec_deciduous_(&nday,&ncycles,&(parValues[0]),drivers,model);

  /* Create band-diagonal covariance matrix */
  int ml  = NDIAG, mu = NDIAG;
  int lda = 2*ml+mu+1;
  int n   = dayen-dayst+1;
  int info, job=0;

  double lcorr2= chain(chainDim-1)*chain(chainDim-1);
  double sigm2 = exp(chain(chainDim-2))*exp(chain(chainDim-2));
  double *factrs = new double[NDIAG];
  for (int i=1;i<=NDIAG;i++)
    factrs[i-1] = exp(-(i*i)/lcorr2)*sigm2;

  double *rhs  = new double[n];
  double *abd  = new double[lda*n];
  int    *ipvt = new int[n];

  for ( int i = 0; i<n; i++ ) 
    rhs[i] = sigm2+obssigma[dayst+i]*obssigma[dayst+i];
  setabd_(abd,rhs,factrs,&lda,&ml,&mu,&n);
  delete []factrs ;

  /* Factor matrix */
  dgbfa_(abd,&lda,&n,&ml,&mu,ipvt,&info);

  /* Solver Sigma^{-1}y */
  for ( int i = 0; i<n; i++ ) 
    rhs[i] = chain(chainDim-3)+model[6*(i+dayst)]-obsdata[i+dayst];
  dgbsl_(abd,&lda,&n,&ml,&mu,ipvt,rhs,&job);

  /* Evaluate likelihood */
  double loglik = 0;
  for ( int i = 0; i<n; i++ ) 
    loglik -= (chain(chainDim-3)+model[6*(i+dayst)]-obsdata[i+dayst])*rhs[i];
  loglik *= 0.5;

  double dsum1 = 0.0;
  for ( int i = 0; i<n; i++ ) 
    dsum1 += log(sigm2+obssigma[dayst+i]*obssigma[dayst+i]);
  loglik -= (0.5*n*log(DPI) + 0.5*dsum1);

  double logPr = logDalecPrior(chain,params);
#ifdef DEBUG
  std::cout<<"Out of logDalecPost: "<<loglik<<std::endl<<std::flush;
#endif

  delete []rhs  ;
  delete []abd  ;
  delete []ipvt ;

  return ( loglik+logPr );

}
#endif

