#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <getopt.h>

#define DPI 6.28318531

void usage() {
  return;
}

float *read_points(const char* filename, int *rows, int ndim)
{
  int i,j;
  float *data, *p, dtmp;
  FILE* fin;

  fin = fopen(filename,"r");
  if (!fin) {
    printf("Cannot open input file: %s\n",filename);
    exit(1);
  }
  *rows=0;
  while (fscanf(fin, "%g", &dtmp) != EOF) {
    for (j=1;j<ndim;++j)
      fscanf(fin,"%g ",&dtmp);
      *rows += 1;
  }
  fclose(fin);
  printf("There are %d rows in the file\n",*rows);

  fin = fopen(filename,"r");
  data = (float*) malloc((*rows)*ndim*sizeof(float));
  if (!data) {
    printf("Cannot allocate memory for %dx%d matrix\n",*rows,ndim);
    exit(1);
  }
  p = data;
  for (i=0;i<(*rows);++i) {
    for (j=0;j<ndim;++j) {
      fscanf(fin,"%g ",p);
      p++;
    }
  }
  fclose(fin);
    
  return data;
}

double minval(float *data, int rows, int ndim, int icol) {
    int i,idx;
    float minv;
    minv = data[icol];
    for (i=1; i<rows; i++) {
        idx = i*ndim+icol;
        if (data[idx]<minv) minv=data[idx];
    }
    return minv;
}

double maxval(float *data, int rows, int ndim, int icol) {
    int i,idx;
    float maxv;
    maxv = data[icol];
    for (i=1; i<rows; i++) {
        idx = i*ndim+icol;
        if (data[idx]>maxv) maxv=data[idx];
    }
    return maxv;
}

float stdev(float *data, int rows, int ndim, int icol)
{
    int i;
    float mean,sumstdev;

    mean = data[icol];
    for (i=1; i<rows; i++)
        mean += data[i*ndim+icol];
    mean = mean/rows;

    sumstdev = (data[icol]-mean)*(data[icol]-mean);   
    for (i=1; i<rows; i++)
        sumstdev += pow(data[i*ndim+icol]-mean,2);

    return sqrt(sumstdev/rows);           
}

int main(int argc, char** argv)
{
  int i, j, k, idx, rows, ndim=0, npts=0;

  float  xmin, xmax, ymin, ymax, x, y, sig1, sig2, facs, dtrh=7.0, dtrh2;
  float *dataset, *pdfD;
  char *filepts=(char *)"NONE";
  char *fileout=(char *)"dens.tar";

  int c;
  while ((c=getopt(argc,(char **)argv,"hd:n:f:o:"))!=-1){
    switch (c) {
    case 'h':
      usage();
      return (0);
      break;
    case 'd':
      ndim = strtol(optarg, (char **)NULL,0);
      break;
    case 'n':
      npts = strtol(optarg, (char **)NULL,0);
      break;
    case 't':
      dtrh = strtof(optarg, (char **)NULL);
      break;
    case 'f':
      filepts = optarg;
      break;
    case 'o':
      fileout = optarg;
      break;
   }
  }

  /* Threshold squared radius */
  dtrh2 = dtrh*dtrh;

  /* Dataset */
  dataset = read_points(filepts, &rows, ndim);

  /* Silverman bandwidth factor */
  facs=pow(rows*(ndim+2.0)/4.0,-1.0/(ndim+4.0));   

  /* Get min/max values */
  xmin = minval(dataset, rows, ndim, 0);
  xmax = maxval(dataset, rows, ndim, 0);
  sig1 = stdev (dataset, rows, ndim, 0);
  if (ndim==2) {
    ymin = minval(dataset, rows, ndim, 1);
    ymax = maxval(dataset, rows, ndim, 1);
    sig2 = stdev (dataset, rows, ndim, 1);
  }
  if (ndim==1) {
    printf("Dataset bounds: [%g,%g]\n",xmin,xmax);
    printf("Dataset bdwths: [%g]x%g\n",sig1,facs);
    sig1 *= facs;
  }
  else {
    printf("Dataset bounds: [%g,%g]x[%g,%g]\n",xmin,xmax,ymin,ymax);
    printf("Dataset bdwths: [%g,%g]x%g\n",sig1,sig2,facs);
    sig1 *= facs; sig2 *= facs;
  }


  float pdfsum = 0.0;
  FILE *fout = fopen(fileout,"w");
  if (ndim==1) {
    pdfD = (float*) malloc(npts*sizeof(float));
    for (i=0; i<npts; i++) {

      x = xmin+(xmax-xmin)*i/(npts-1);
      pdfD[i] = 0.0;
      for (k=0; k<rows; k++) {
        float d2 = pow((dataset[k]-x)/sig1,2);
        if (d2<dtrh2) pdfD[i] += exp(-0.5*d2);
      }
      pdfD[i] /= (sqrt(DPI)*sig1*rows);
      pdfsum += pdfD[i];
    }
    pdfsum *= (xmax-xmin)/(npts-1);
    for (i=0; i<npts; i++) {
      x = xmin+(xmax-xmin)*i/(npts-1);
      fprintf(fout,"%g %g\n",x,pdfD[i]/pdfsum);
    }
  }
  else {
    pdfD  = (float*) malloc(npts*npts*sizeof(float));
    for (i=0; i<npts; i++) {

      x = xmin+(xmax-xmin)*i/(npts-1);
      //printf("%d\n",i);

      for (j=0; j<npts; j++) {

        y = ymin+(ymax-ymin)*j/(npts-1);

        idx = i*npts+j;
        pdfD[idx] = 0.0;
        for (k=0; k<rows; k++) {
          float d2 = pow((dataset[k*2]-x)/sig1,2)+pow((dataset[k*2+1]-y)/sig2,2);
          if (d2<dtrh2) pdfD[idx] += exp(-0.5*d2);
        }
        pdfD[idx] /= (DPI*sig1*sig2*rows);
        pdfsum += pdfD[idx];
      }
    }
    pdfsum *= (xmax-xmin)/(npts-1);
    pdfsum *= (ymax-ymin)/(npts-1);
    for (j=0; j<npts; j++) {
      y = ymin+(ymax-ymin)*j/(npts-1);
      for (i=0; i<npts; i++) {
        x = xmin+(xmax-xmin)*i/(npts-1);
        fprintf(fout,"%g %g %g\n",x,y,pdfD[i*npts+j]/pdfsum);
      }
    }
  }
  fclose(fout);

  free(dataset);
  free(pdfD);
 
  return (0);

}

