#!/usr/bin/env python

import os, sys
import numpy as npy
import h5py
import matplotlib.pyplot as plt
from   scipy.stats import norm,lognorm

din=npy.load('dst_d12/Dkl_18.npz')
Dkl_18=din['Dkl']
pnm_18=din['pnames']
Dkl_18=npy.concatenate((Dkl_18,npy.array([0.0]*5)))
pnm_18=npy.concatenate((pnm_18,npy.array(['vc1','vc2','vc3','sc1','sc2'])))

din=npy.load('dtr_d12/Dkl_23.npz')
Dkl_23=din['Dkl']
pnm_23=din['pnames']

# sort things
pnm_23 = pnm_23[npy.argsort(Dkl_23)]
Dkl_23.sort()
pnm_18 = pnm_18[npy.argsort(Dkl_18)]
Dkl_18.sort()

# Delete parameters with negligible Dkl
pnopl=npy.intersect1d(pnm_18[npy.where(Dkl_18<0.5)],pnm_23[npy.where(Dkl_23<0.5)])
idx18 = npy.array([npy.where(pnm_18==i)[0] for i in pnopl]).flatten()
pnm_18=npy.delete(pnm_18,idx18)
Dkl_18=npy.delete(Dkl_18,idx18)
idx23 = npy.array([npy.where(pnm_23==i)[0] for i in pnopl]).flatten()
pnm_23=npy.delete(pnm_23,idx23)
Dkl_23=npy.delete(Dkl_23,idx23)

# Sort 
idx=npy.array([npy.where(pnm_18==i)[0] for i in pnm_23]).flatten()
Dkl_18=Dkl_18[idx]
pnm_18=pnm_18[idx]
npar=Dkl_23.shape[0]

width = 0.8
ind = npy.arange(npar)+0.5-width/2.0
fs1 = 24

fig=plt.figure(figsize=(12,6))
ax=fig.add_axes([0.10,0.23,0.87,0.72])
br2 = ax.bar(ind+width/2, Dkl_18, width/2, color='b',label=r'$D_{ST}$')
br1 = ax.bar(ind, Dkl_23, width/2, color='r',label=r'$D_{TR}$')

ax.set_xlim([0,npar])
xtm=ax.set_xticks(ind+width/2)
xtl=ax.set_xticklabels(['$'+v+'$' for v in pnm_23],rotation=60,fontsize=fs1-2);

ax.set_ylim([0,16])
ax.set_yticks([0,4,8,12,16])
ax.set_ylabel(r'$D_{KL}(p\vert\vert q)$',fontsize=fs1)

for tick in ax.yaxis.get_major_ticks():
    tick.label.set_fontsize(fs1-4)

plt.legend(loc="upper left",prop={'size':fs1})

#plt.show()
plt.savefig('Dkl.pdf')
