#!/usr/bin/env python

import sys, os
import numpy as npy
import h5py
import matplotlib.pyplot as plt
from scipy import stats, mgrid, c_, reshape, random, rot90

if sys.argv[1] == '18':
    filename='pp_18.pdf'
else:
    filename='pp_23.pdf'

#-load posterior predictive
if sys.argv[2] == "pp":
    din=npy.load("ppmod_pderr.npz")
else:
    din=npy.load("ppmod.npz")

ppmod=din["ppmod"]
nmev=ppmod.shape[0]

#-load observations 
ierr=os.system("awk 'NR>1{print $0}' US-Ha1_NEE_daily.txt > daily.txt")
obs=npy.genfromtxt("daily.txt")
dayst=366
dayen=3365
ndays = dayen-dayst+1
nee = obs[dayst:dayen+1,2]
neesig = obs[dayst:dayen+1,4]

#-quantiles
model05 = npy.zeros(ndays)
model25 = npy.zeros(ndays)
model75 = npy.zeros(ndays)
model95 = npy.zeros(ndays)
modelmn = npy.zeros(ndays)
for j in range(ndays):
    model05[j] = npy.percentile(ppmod[:,j], 5.0)
    model25[j] = npy.percentile(ppmod[:,j],25.0)
    model75[j] = npy.percentile(ppmod[:,j],75.0)
    model95[j] = npy.percentile(ppmod[:,j],95.0)
    modelmn[j] = npy.mean(ppmod[:,j])


#-set days/months
dayno = npy.array([i+1 for i in range(ndays)])
mos=npy.array([732, 763, 791, 822, 852, 883, 913, 944, 975, 1005, 1036, 1066,
              1097, 1128, 1156, 1187, 1217, 1248, 1278, 1309, 1340, 1370, 1401, 1431,
              1462, 1493, 1522, 1553, 1583, 1614, 1644, 1675, 1706, 1736, 1767, 1797])
mon=["J","F","M","A","M","J","J","A","S","O","N","D"]

#-make figure
fs1=16
fig = plt.figure(figsize=(10,4))
ax=fig.add_axes([0.10,0.05,0.88,0.9])

pl3=plt.fill_between(dayno, model05, model95,linewidth=1, alpha=0.5, edgecolor='#1B2ACC', facecolor='#089FFF',label="1-99%")
pl2=plt.fill_between(dayno, model25, model75,linewidth=1, alpha=0.5, edgecolor='green', facecolor='#00FA9A',label="25-75%")
pl1=plt.plot(dayno,obs[dayst:dayen+1,2],linewidth=1,color="red",label="US-Ha1")

xlm=ax.set_xlim([800,1500])
ylm=ax.set_ylim([-12,10])
xt=ax.set_xticks([1097,1462])
xtl=ax.set_xticklabels([])
tx1 = plt.text(900,  -11, '1994', fontsize=fs1+4)
tx2 = plt.text(1280, -11, '1995', fontsize=fs1+4)
xts=ax.xaxis.set_tick_params(width=3,size=12)

# draw months
for i in range(len(mos)-1):
    j = i%12
    if (mos[i] > 800) and (mos[i+1]<1500):
        plt.plot([mos[i],mos[i]],[8.5,10],lw=0.5,color='k')
        plt.plot([mos[i],mos[i]],[-12,-11],lw=0.5,color='k')
        txm = plt.text(0.5*(mos[i]+mos[i+1]), 9, mon[j], fontsize=fs1,ha='center',va='center')

#xl=ax.set_xlabel(r"day since 1/1/1992",fontsize=16)
yl=ax.set_ylabel(r"NEE $[gC\cdot m^{-2}\cdot day^{-1}]$",fontsize=16)
#ax.set_title("Posterior predictive check",fontsize=16)
#plt.legend(loc="upper right")
#plt.show()
plt.savefig(filename)




