#=====================================================================================
#                     The UQ Toolkit (UQTk) version 2.1
#                    Copyright (2013) Sandia Corporation
#                     http://www.sandia.gov/UQToolkit/
#
#    Copyright (2013) Sandia Corporation. Under the terms of Contract DE-AC04-94AL85000
#    with Sandia Corporation, the U.S. Government retains certain rights in this software.
#
#    This file is part of The UQ Toolkit (UQTk)
#
#    UQTk is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    UQTk is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with UQTk.  If not, see <http://www.gnu.org/licenses/>.
#
#    Questions? Contact Bert Debusschere <bjdebus@sandia.gov>
#    Sandia National Laboratories, Livermore, CA, USA
#====================================================================================
try:
    import os
except ImportError:
    print "gsa library requires the os package -> Exit"
    quit()

try:
    import numpy as npy
except ImportError:
    print "gsa library requires numpy package -> Exit"
    quit()

def genSpl_Si(npar,verb=0):
    # get optional arguments
    splout  = "gsaSplSi.dat" # samples file
    nd      = 12             # no. of significant digits in samples output
    #------------------------------------------------------------------------------------
    # create nspl uniform samples in [a_i,b_i], i=1,ndim
    #------------------------------------------------------------------------------------
    mat1=npy.genfromtxt('sgsa1.dat')
    mat2=npy.genfromtxt('sgsa2.dat')
    # Check dimensions
    if ( (mat1.shape[0] != mat2.shape[0]) or (mat1.shape[0] != mat2.shape[0]) ):
        print "Something wrong with shapes of matrices with samples:"
        print mat1.shape," vs ",mat2.shape
    # assemble the big matrix for main sensitivities
    if npar==18:
        indices=[[0],[1],[2],[3],[4],[5,6],[7],[9],[10],[11],[12],[13],[14],[15],[16],[17],[18]]
    else:
        indices=[[0],[1],[2],[3],[4],[5,6],[7],[9],[10],[11],[12],[13],[14],[15],[16],[17],[18],[19],[20],[21],[22],[23]]
    if os.path.isfile(splout):
        os.remove(splout)
    f_handle = file(splout, 'a')
    npy.savetxt(f_handle, mat1, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    for idim in range(len(indices)):
        if verb==1:
            print "    - working on parameters:",indices[idim]
        matj=mat2.copy();
        matj[:,indices[idim]]=mat1[:,indices[idim]]
        npy.savetxt(f_handle, matj, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    npy.savetxt(f_handle, mat2, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    f_handle.close()
    return

def genSens_Si(ymod,nspl):
    verb=0
    ndim = ymod.shape[0]/nspl-2
    print "No of params: ",ndim
    if verb > 0:
        print "Compute sensitivities, no. of samples:",nspl
    sobolSi = npy.zeros(ndim)
    yMat1   = ymod[:nspl]
    yMat2   = ymod[nspl*(ndim+1):]
    mean12  = npy.mean(yMat1*yMat2)
    vv1     = npy.var(yMat1,ddof=1)
    for idim in range(1,ndim+1):    
        vari = npy.sum(yMat1*ymod[idim*nspl:(idim+1)*nspl])
        sobolSi[idim-1] = (vari/(nspl-1.0)-mean12)/vv1
        if verb > 1:
            print " - parameter ",idim,": ",sobolSi[idim-1]
    if verb > 0:
        print " - total first order sensitivity: ",npy.sum(sobolSi)
    return sobolSi

def genSpl_SiT(npar,verb=0):
    # get optional arguments
    splout  = "gsaSplSiT.dat" # samples file
    nd      = 12              # no. of significant digits in samples output
    #------------------------------------------------------------------------------------
    # create nspl uniform samples in [a_i,b_i], i=1,ndim
    #------------------------------------------------------------------------------------
    mat1=npy.genfromtxt('sgsa1.dat')
    mat2=npy.genfromtxt('sgsa2.dat')
    # Check dimensions
    if ( (mat1.shape[0] != mat2.shape[0]) or (mat1.shape[0] != mat2.shape[0]) ):
        print "Something wrong with shapes of matrices with samples:"
        print mat1.shape," vs ",mat2.shape
    # assemble the big matrix for main sensitivities
    if npar==18:
        indices=[[0],[1],[2],[3],[4],[5,6],[7],[9],[10],[11],[12],[13],[14],[15],[16],[17],[18]]
    else:
        indices=[[0],[1],[2],[3],[4],[5,6],[7],[9],[10],[11],[12],[13],[14],[15],[16],[17],[18],[19],[20],[21],[22],[23]]
    if os.path.isfile(splout):
        os.remove(splout)
    f_handle = file(splout, 'a')
    npy.savetxt(f_handle, mat1, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    for idim in range(len(indices)):
        print " - working on parameters:",indices[idim]
        matj=mat1.copy();
        matj[:,indices[idim]]=mat2[:,indices[idim]]
        npy.savetxt(f_handle, matj, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    npy.savetxt(f_handle, mat2, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    f_handle.close()
    return

def genSens_SiT(ymod,nspl):
    verb=0
    ndim = ymod.shape[0]/nspl-2
    print "No of params: ",ndim
    if verb > 0:
        print "Compute sensitivities, no. of samples:",nspl
    sobolSiT = npy.zeros(ndim)
    yMat1  = ymod[:nspl]
    vv1    = npy.var(yMat1,ddof=1)
    Ey     = npy.average(yMat1)
    for idim in range(1,ndim+1):
        vari      = npy.sum(npy.power(yMat1-ymod[idim*nspl:(idim+1)*nspl],2))/nspl
        sobolSiT[idim-1] = vari/(2.0*vv1);
        if verb > 1:
            print " - parameter ",idim,": ",sobolSiT[idim-1]
    if verb > 0:
        print " - total main sensitivity: ",npy.sum(sobolSiT)
    return sobolSiT

def genSpl_Sij(npar,verb=0):
    verb = 2
    splout  = "gsaSplSij.dat" # samples file
    nd      = 12              # no. of significant digits in samples output
    mat1=npy.genfromtxt('sgsa1.dat')
    mat2=npy.genfromtxt('sgsa2.dat')
    # assemble the big matrix for main sensitivities
    if npar==18:
        indices=[[0],[1],[2],[3],[4],[5,6],[7],[9],[10],[11],[12],[13],[14],[15],[16],[17],[18]]
    else:
        indices=[[0],[1],[2],[3],[4],[5,6],[7],[9],[10],[11],[12],[13],[14],[15],[16],[17],[18],[19],[20],[21],[22],[23]]
    # assemble the big matrix for main sensitivities
    if os.path.isfile(splout):
        os.remove(splout)
    f_handle = file(splout, 'a')
    npy.savetxt(f_handle, mat1, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    for idim in range(len(indices)-1):
        for jdim in range(idim+1,len(indices)):
            if verb>1:
                print " - working on pair ",idim,jdim
            matj=mat2.copy();
            matj[:,indices[idim]]=mat1[:,indices[idim]]
            matj[:,indices[jdim]]=mat1[:,indices[jdim]]
            npy.savetxt(f_handle, matj, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    npy.savetxt(f_handle, mat2, fmt="%."+str(nd)+"e", delimiter='  ', newline='\n')
    f_handle.close()
    return

def genSens_Sij(sobolSi,ymod,nspl):
    verb=0
    #------------------------------------------------------------------------------------
    # joint sensitivities
    #------------------------------------------------------------------------------------
    ndim = len(sobolSi)
    if (nspl*(ndim*(ndim-1)/2+2) != ymod.shape[0]):
        print "Something wrong with the number of model evaluations"
        return
    if verb > 0:
        print "No. of samples, no. of dimensions:",nspl,ndim
    sobolSij = npy.array(npy.zeros((ndim,ndim)))
    yMat1    = ymod[:nspl]
    yMat2    = ymod[nspl*(ndim*(ndim-1)/2+1):]
    mean12   = npy.mean(yMat1*yMat2)
    vv1      = npy.var(yMat1,ddof=1)
    ijd = 0
    for idim in range(1,ndim):    
        for jdim in range(idim+1,ndim+1):
            ijd += 1;
            vari = npy.sum(yMat1*ymod[ijd*nspl:(ijd+1)*nspl])
            sobolSij[idim-1,jdim-1] = (vari/(nspl-1.0)-mean12)/vv1-sobolSi[idim-1]-sobolSi[jdim-1]
            if abs(sobolSij[idim-1,jdim-1])<1.e-4:
                sobolSij[idim-1,jdim-1]=0.0;
            if verb > 1:
                print " - pair ",idim,jdim,": ",sobolSij[idim-1,jdim-1]
    if verb > 0:
        print " - total Sij: ",npy.sum(sobolSij[:,:]),sobolSij[:,:].max(),sobolSij[:,:].argmax()
    return sobolSij





